//-----------------------------------------------------------------------------
// F320_USB0_Kbd.c
//-----------------------------------------------------------------------------
// Copyright 2014 Silicon Laboratories, Inc.
// http://developer.silabs.com/legal/version/v11/Silicon_Labs_Software_License_Agreement.txt
//
// Program Description:
//
// Source file for routines that mouse data.
//
//
// How To Test:    See Readme.txt
//
//
// FID:
// Target:         C8051F320
// Tool chain:     Keil / Raisonance
//                 Silicon Laboratories IDE version 2.6
// Command Line:   See Readme.txt
// Project Name:   F3xx_MouseExample
//
// Release 1.3
//    -All changes by BW
//    -1 SEP 2010
//    -Updated USB0_Init() to write 0x89 to CLKREC instead of 0x80
// Release 1.2 (ES)
//    -Added support for Raisonance
//    -No change to this file
//    -02 APR 2010
// Release 1.1
//    -Minor code comment changes
//    -16 NOV 2006
// Release 1.0
//    -Initial Revision (PD)
//    -07 DEC 2005
//

#include <string.h>
#include "c8051f3xx.h"
#include "F320_USB0_KBD.h"
#include "F3xx_USB0_InterruptServiceRoutine.h"
#include "F3xx_USB0_Register.h"
#include "F3xx_USB0_ReportHandler.h"
#include "F32x_SMBus_EEPROM.h"

//-----------------------------------------------------------------------------
// Definitions
//-----------------------------------------------------------------------------

// USB clock selections (SFR CLKSEL)
#define USB_4X_CLOCK       0x00        // Select 4x clock multiplier, for USB
#define USB_INT_OSC_DIV_2  0x10        // Full Speed
#define USB_EXT_OSC        0x20
#define USB_EXT_OSC_DIV_2  0x30
#define USB_EXT_OSC_DIV_3  0x40
#define USB_EXT_OSC_DIV_4  0x50

// System clock selections (SFR CLKSEL)
#define SYS_INT_OSC        0x00        // Select to use internal oscillator
#define SYS_EXT_OSC        0x01        // Select to use an external oscillator
#define SYS_4X_DIV_2       0x02


//-----------------------------------------------------------------------------
// Local Definitions
//-----------------------------------------------------------------------------


//-----------------------------------------------------------------------------
// Global Variable Declarations
//-----------------------------------------------------------------------------

sKey Keys[12]; //Programmed key codes
/* A keystroke input report. */
unsigned char KeyReport[8];
volatile unsigned char IdleTime, idleRate = DEFAULT_IDLE_RATE;


//-----------------------------------------------------------------------------
// Local Variable Declarations
//-----------------------------------------------------------------------------



//-----------------------------------------------------------------------------
// Local Function Prototypes
//-----------------------------------------------------------------------------

void Sysclk_Init (void);
void Port_Init (void);
void USB0_Init (void);
void Timer2_Init (void);
void Delay (void);


//-----------------------------------------------------------------------------
// Interrupt Service Routines
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// Timer2_ISR
//-----------------------------------------------------------------------------
//
// Called when Timer 2 overflows, every 4ms.
//
//-----------------------------------------------------------------------------
INTERRUPT(Timer2_ISR, TIMER2_IRQn)
{
	IdleTime--;	//проверяется на 0 в гл.цикле

   TMR2CN_TF2H = 0;                           // Clear Timer2 interrupt flag
}




//-----------------------------------------------------------------------------
// Initialization Routines
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// System_Init(void)
//-----------------------------------------------------------------------------
//
// Return Value - None
// Parameters - None
//
// This top-level initialization routine calls all support routine.
//
// ----------------------------------------------------------------------------
void System_Init (void)
{
	unsigned char i;

		REG0CN = REG0CN_REG0DIS__DISABLED;  ///< Disable the VREG0 Voltage Regulator. На плате есть IRU1010-33
   PCA0MD &= ~0x40;                    // Disable Watchdog timer
   Sysclk_Init ();                     // initialize system clock

   // If slave is holding SDA low because of an improper SMBus reset or error
#ifndef TEST
   while(!SDA)
   {
      // Provide clock pulses to allow the slave to advance out
      // of its current state. This will allow it to release SDA.
      XBR1 = 0x40;                     // Enable Crossbar
      SCL = 0;                         // Drive the clock low
      for(i = 0; i < 255; i++);        // Hold the clock low
      SCL = 1;                         // Release the clock
      while(!SCL);                     // Wait for open-drain
                                       // clock output to rise
      for(i = 0; i < 10; i++);         // Hold the clock high
      XBR1 = 0x00;                     // Disable Crossbar
   }
#endif
   Port_Init ();                       // configure cross bar
#ifndef TEST
   Timer1_Init ();                     // Configure Timer1 for use as SMBus clock source
#endif
	Timer2_Init ();

#ifndef TEST
   Timer3_Init ();                     // Configure Timer3 for use with SMBus
                                       // low timeout detect

   SMBus_Init ();                      // Configure and enable SMBus
#endif
}

//-----------------------------------------------------------------------------
// USB0_Init
//-----------------------------------------------------------------------------
//
// Return Value - None
// Parameters - None
//
// - Initialize USB0
// - Enable USB0 interrupts
// - Enable USB0 transceiver
// - Enable USB0 with suspend detection
//
// ----------------------------------------------------------------------------

void USB0_Init (void)
{

   POLL_WRITE_BYTE (POWER, 0x08);      // Force Asynchronous USB Reset
   POLL_WRITE_BYTE (IN1IE, 0x07);      // Enable Endpoint 0-1 in interrupts
   POLL_WRITE_BYTE (OUT1IE,0x07);      // Enable Endpoint 0-1 out interrupts
   POLL_WRITE_BYTE (CMIE, 0x07);       // Enable Reset, Resume, and Suspend
                                       // interrupts
   USB0XCN = 0xE0;                     // Enable transceiver; select full speed
   POLL_WRITE_BYTE (CLKREC,0x89);      // Enable clock recovery, single-step
                                       // mode disabled

   EIE1 |= 0x02;                       // Enable USB0 Interrupts

                                       // Enable USB0 by clearing the USB
   POLL_WRITE_BYTE (POWER, 0x01);      // Inhibit Bit and enable suspend
                                       // detection

}


//-----------------------------------------------------------------------------
// Sysclk_Init
//-----------------------------------------------------------------------------
//
// Return Value - None
// Parameters - None
//
// Initialize system clock to maximum frequency.
//
// ----------------------------------------------------------------------------
void Sysclk_Init (void)
{

   OSCICN = 0x83;                     // Configure internal oscillator for
                                       // its maximum frequency and enable
                                       // missing clock detector

#ifdef _USB_LOW_SPEED_
   CLKSEL  = SYS_INT_OSC | USB_INT_OSC_DIV_2;              // Select System & USB clocks
#else
   CLKMUL  = 0x00;                     // Select internal oscillator as
                                       // input to clock multiplier

   CLKMUL |= 0x80;                     // Enable clock multiplier
   Delay();                            // Delay for clock multiplier to begin
   CLKMUL |= 0xC0;                     // Initialize the clock multiplier
   Delay();                            // Delay for clock multiplier to begin

   while(!(CLKMUL & 0x20));            // Wait for multiplier to lock
   CLKSEL  = SYS_INT_OSC;              // Select system clock
   CLKSEL |= USB_4X_CLOCK;             // Select USB clock
#endif  /* _USB_LOW_SPEED_ */
}

//-----------------------------------------------------------------------------
// Port_Init
//-----------------------------------------------------------------------------
//
// Return Value - None
// Parameters - None
//
// - Configure the Crossbar and GPIO ports.
//
// ----------------------------------------------------------------------------
void Port_Init (void)
{
   //P0MDOUT = 0x00;                    // Port 0 ALL pins set open-drain.
	P0MDOUT = 0x40;                    // Port 0 pin 6 set push-pull
   P2MDOUT = 0x0F;                    // Port 2 pins 0-3 set push-pull
   XBR0     = 0x04;										// Bit 2 - SMBus I/O
   XBR1     = 0x40;                   // Enable Crossbar, Port I/O Weak Pull-up
   P0 = 0xFF;
}

//-----------------------------------------------------------------------------
// Timer2_Init
//-----------------------------------------------------------------------------
//
// Return Value - None
// Parameters - None
//
// - Timer 2 reload, used for the idleRate timer.
// Rate unit is 4 ms.
//
// ----------------------------------------------------------------------------
void Timer2_Init (void)
{
   TMR2CN  = 0x00;                     // Stop Timer2; Clear TF2;
   CKCON &= ~CKCON_T2ML__BMASK;                    // Timer2 clocked SYSCLK / 12 based on TMR2CN_T2XCLK=0 
   TMR2RL  = 61536;                        // Initialize reload value 65536−4000 for 4ms period
   TMR2    = TMR2RL;

   IE_ET2     = 1;                        // Enable Timer2 interrupts
   TMR2CN_TR2     = 1;                        // Start Timer2
}

//-----------------------------------------------------------------------------
// Delay
//-----------------------------------------------------------------------------
//
// Return Value - None
// Parameters - None
//
// Used for a small pause, approximately 80 us in Full Speed,
// and 1 ms when clock is configured for Low Speed
//
// ----------------------------------------------------------------------------
void Delay (void)
{
   int x;
   for (x = 0; x < 500; x)
      x++;
}

//-----------------------------------------------------------------------------
// Main Routine
//-----------------------------------------------------------------------------
void main(void)
{
	unsigned char c0 = 0xFF; //Номер кнопки от предыдущего нажатия
	unsigned char c; //Номер нажатой кнопки (FF - отжатие)
	unsigned char MxCh;//канал мультиплексора

	 IN_BUFFER.Ptr = KeyReport;
	 IN_BUFFER.Length = sizeof(KeyReport);
 IdleTime = idleRate;
   System_Init ();

	memset(KeyReport, 0, sizeof(KeyReport));
	memset(Keys, 0, sizeof(Keys));

   IE_EA = 1;

#ifdef TEST
        Keys[0].ID = 0x1E; //Код кнопки "1"
        for (c=1; c<12; c++) Keys[c].ID = Keys[0].ID + c; //Код кнопки
#else
	//Чтение в 2 этапа, т.к. массив больше страницы.
	EEPROM_ReadArray(Keys, 0, SMB_BUFF_SIZE);
	EEPROM_ReadArray(Keys, SMB_BUFF_SIZE, sizeof(Keys)-SMB_BUFF_SIZE);
#endif

   USB0_Init ();
	 
L_MainCyc:
  while (1)
   {
      if (USB0_STATE == DEV_CONFIGURED)	// If the device is configured, then
	{
      if (idleRate && !IdleTime)	// If Idle timer is active and expired
			{
               SendPacket();
		IdleTime = idleRate;
		}
    	  //Опрoс клавиатуры
				c = 0;
          for (MxCh=0; MxCh<16; MxCh++) {
							if (MxCh==9) MxCh++; else if (MxCh==11) MxCh = 14; //Пропуск неподключенных каналов 9, 11-13
              P2 = MxCh | 0xF0; //канал мультиплексора
              Delay();
              if (!MUX_X)      //0 на канале = кнопка нажата
              {
               if (c0 == c) goto L_MainCyc; //без изменения
               KeyReport[0] = Keys[c].Shifts;
               KeyReport[2] = Keys[c].ID;
               c0 = c;      //запомнить состояние
               SendPacket();
               goto L_MainCyc;
              }
							c++;
          }

    	  //Cюда доходит, когда не нажато ничего
          if (c0 != 0xFF) {
					KeyReport[0] = 0;
					KeyReport[2] = 0;
                                        c0 = 0xFF;
    		  SendPacket();
    	  }
	}
   }
}

